#!/bin/sh

# This script will run tests with the update script to ensure
# that all is good :)

SCRIPT=/home/mish/active/indymedia/mirror-scripts/update.pl

TESTDIR=/tmp/test_updatemirror
CHANGEDIR=$TESTDIR/changes
STATEDIR=$TESTDIR/state

LASTUPDATEFILE=$STATEDIR/lastupdate.txt
OUTPUTFILE=$TESTDIR/out.txt
CORRECTFILE=$TESTDIR/correct.txt

# set up - set up the directories and files required
setup ()
{
	# first remove anything left
	rm -rf $TESTDIR

	# create the directory and copy the files we need to it
	mkdir -p $CHANGEDIR
	mkdir -p $STATEDIR
	cp change*.txt $TESTDIR/
	return
}

# teardown - remove all stuff so that next time is clean
teardown ()
{
	# do we want to wait before tear down
	if [ "$INTERACTIVE" == "i" ]; then
		echo "Press any key to continue to next test"
		read -n1
	fi

	rm -rf $TESTDIR
	return
}

# setlastupdate - set the last update date/time
# pass the date/time as YYYY MM DD hh mm ss
setlastupdate ()
{
	# set the format as for `date -s %Y/%m/%d %H:%M:%S`
	# or use that if no argument is set
	if [ "$#" -eq "0" ]; then
		date +"%Y:%m:%dT%H:%M:%S" > $LASTUPDATEFILE
	else
		echo "$1:$2:$3T$4:$5:$6" > $LASTUPDATEFILE
	fi
	return
}

# get the list of change files - one day only
testChangeFileListOneDay ()
{
	setup 
	setlastupdate 2006 06 22 16 13 11
	$SCRIPT --test \
		--workingdir=$TESTDIR \
		--list-change-files \
		--now="2006:06:22T16:31:11.01" \
		--output=$OUTPUTFILE
	# check output file
	echo "/changes/changes20060622.txt" > $CORRECTFILE
	diff $OUTPUTFILE $CORRECTFILE
	if [ "$?" -ne "0" ]; then
		echo "*** FAILED testChangeFileListOneDay ***"
	else
		echo "Passed testChangeFileListOneDay"
	fi
 
	teardown 
}


# generic test 
testGeneric ()
{
	setup 
	teardown 
}

# do we want to do tests interactively
if [ $# -eq 0 ]; then
	INTERACTIVE=0
else
	INTERACTIVE=$1
fi


# do the tests
testGeneric
testChangeFileListOneDay 
