/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.generator.tal;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import mir.generator.tal.interfaces.TALExpressionParser;
import mir.generator.Generator;
import mir.util.ParameterExpander;
import mir.util.RewindableIterator;

public class MirExpressionParser implements TALExpressionParser {
  public Object preparseExpression(String anExpression) {
    return anExpression;
  }
  public Object preparseStringExpression(String anExpression)  {
    return anExpression;
  }

  public Object preparseBooleanExpression(String anExpression) {
    return anExpression;
  }

  public Object preparseReferenceExpression(String anExpression) {
    return anExpression;
  }

  public Object preparseListExpression(String anExpression) {
    return anExpression;
  }

  public Object preparseTRUE() {
    return "1==1";
  }

  public Object preparseFALSE() {
    return "0==1";
  }

  public Object evaluateExpression(Object aContext, Object aPreparsedExpression) {
    try {
      return ParameterExpander.evaluateExpression(aContext, (String) aPreparsedExpression);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public String evaluateStringExpression(Object aContext, Object aPreparsedExpression) {
    try {
      return ParameterExpander.expandExpression(aContext, (String) aPreparsedExpression);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public boolean evaluateBooleanExpression(Object aContext, Object aPreparsedExpression) {
    try {
      return ParameterExpander.evaluateBooleanExpression(aContext, (String) aPreparsedExpression);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public Iterator evaluateListExpression(Object aContext, Object aPreparsedExpression) {
    try {
      Object list = ParameterExpander.evaluateExpression(aContext, (String) aPreparsedExpression);

      if (list instanceof List)
        return ((List) list).iterator();

      if (list instanceof RewindableIterator) {
        ((RewindableIterator) list).rewind();
        return (RewindableIterator) list;
      }

      if (list instanceof Iterator) {
        return (Iterator) list;
      }

      throw new TALExc("Not a list: " + list);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public void processAssignment(Object aContext, Object aPreparsedReferenceExpression, Object aPreparsedExpression) {
    try {
      ParameterExpander.setValueForKey( (Map) aContext, (String) aPreparsedReferenceExpression, ParameterExpander.evaluateExpression(aContext, (String) aPreparsedExpression));
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public void processDirectAssignment(Object aContext, Object aPreparsedReferenceExpression, Object aValue) {
    try {
      ParameterExpander.setValueForKey( (Map) aContext, (String) aPreparsedReferenceExpression, aValue);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public void processPseudoAssignment(Object aContext, String aName, Object aValue) {
    try {
      ParameterExpander.setValueForKey( (Map) aContext, aName, aValue);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }

  public Object evaluatePseudoVariable(Object aContext, String aName) {
    try {
      return ParameterExpander.getObjectField(aContext, aName);
    }
    catch (Throwable t) {
      throw new TALFailure(t);
    }
  }
}