/*
 * Copyright (C) 2001-2006 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.abuse;

import gnu.regexp.RE;
import mir.entity.Entity;
import mir.session.Request;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;


/**
  * A description of a regular expression filter.
  */
 public class RegularExpressionFilterType extends AbstractFilterType {
    private boolean exactMatch;
    private boolean caseSensitive;
    private int fieldKind;
    private List selectedFields;

    public static final int ENTITY_FIELDS = 0;
    public static final int REQUEST_HEADERS = 2;

    public RegularExpressionFilterType(String aName) {
      this(aName, false, false, null);
    }

    public RegularExpressionFilterType(String aName, boolean aCaseSensitive, boolean anExactMatch, String[] aSelectedFields) {
      this (aName, aCaseSensitive, anExactMatch, ENTITY_FIELDS, aSelectedFields);
    }

    public RegularExpressionFilterType(String aName, boolean aCaseSensitive, boolean anExactMatch, int aFieldKind, String[] aSelectedFields) {
      super(aName);

      fieldKind = aFieldKind;

      caseSensitive = aCaseSensitive;
      exactMatch = anExactMatch;
      if (aSelectedFields==null)
        selectedFields = null;
      else
        selectedFields = Arrays.asList(aSelectedFields);
    }


  public FilterInstance constructFilterInstance(final String anExpression) throws AbuseExc {
    try {
      new RE(anExpression);

      return new FilterInstance() {

        public boolean test(Entity anEntity, Request aRequest) {

          try {
            Iterator j;
            int flags = 0;

            if (!caseSensitive) {
              flags |= RE.REG_ICASE;
            }

            RE regularExpression = new RE(anExpression, flags);

            switch (fieldKind) {
              case REQUEST_HEADERS:
                if (selectedFields != null) {
                  j = selectedFields.iterator();

                  while (j.hasNext()) {
                    String fieldName = (String) j.next();
                    String field = aRequest.getHeader(fieldName);

                    if (exactMatch) {
                      if (field != null && regularExpression.isMatch(field)) {
                        return true;
                      }
                    }
                    else {
                      if (field != null && regularExpression.getMatch(field) != null) {
                        return true;
                      }
                    }
                  }
                }
                break;
              case ENTITY_FIELDS:
                if (selectedFields != null)
                  j = selectedFields.iterator();
                else
                  j = anEntity.getFieldNames().iterator();

                while (j.hasNext()) {
                  String field = anEntity.getFieldValue( (String) j.next());

                  if (exactMatch) {
                    if (field != null && regularExpression.isMatch(field)) {
                      return true;
                    }
                  }
                  else {
                    if (field != null && regularExpression.getMatch(field) != null) {
                      return true;
                    }
                  }
                }
            }
          }
          catch (Throwable t) {
          }
          return false;
        }

        public String status() {
          return null;
        }
      };
    }
    catch (Throwable t) {
      throw new AbuseExc("Invalid expression: " + t.getMessage());
    }
  }
}