/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package  mircoders.media;

import java.io.File;
import java.io.FileOutputStream;

import mir.entity.Entity;
import mir.log.LoggerWrapper;
import mir.media.MediaExc;
import mir.media.MediaFailure;
import mir.media.MediaHandler;
import mir.misc.StringUtil;
import mir.util.IORoutines;

/**
 * Please note: this media handler produces
 * 3 media files, the raw .mp3, a .m3u which is
 * contains the URL for the mp3 and a .pls which
 * contains the URL to the mp3 in shoutcast playlist
 * format. What's important is that the web server (of
 * the media host) must recognize the .m3u and .pls file
 * extensions and send the proper "audio/x-mpegurl"
 * and "audio/x-scpls" mime-types respectively.
 * If the web server is apache, it's easy, just
 * add:
 *
 * audio/x-mpegurl                 m3u
 * audio/x-scpl                    pls
 *
 * to the file pointed to by the "TypesConfig"
 * command in your apache config file. Or add
 * and equivalent AddType command to your httpd.conf.
 * Of course this assumes that the mod_mime is loaded.
 *
 * If the web server is not apache, then your on your own.
 *
 * @see mir.media.MediaHandler
 * @author mh <mh@nadir.org>
 * @version $Id: MediaHandlerMp3.java,v 1.15.2.8 2006/11/12 18:44:46 yossarian Exp $
 */

public class MediaHandlerMp3 extends MediaHandlerAudio implements MediaHandler {
  public MediaHandlerMp3() {
    logger = new LoggerWrapper("Media.Audio.Mp3");
  }

  /**
   * {@inheritDoc}
   */
  public void produce(Entity ent, Entity mediaTypeEnt) throws MediaExc, MediaFailure {
    super.produce(ent, mediaTypeEnt);

    String baseName = ent.getId();
    String date = ent.getFieldValue("date");
    String datePath = StringUtil.webdbDate2path(date);
    String mp3Pointer = getPublishHost() + ent.getFieldValue("publish_path");
    String mpegURLFile = baseName + ".m3u";
    String playlistFile = baseName + ".pls";

    try {
      IORoutines.writeStream(mp3Pointer, "us-ascii",
          new FileOutputStream(new File(getBaseStoragePath() + "/" + datePath + "/" + mpegURLFile)));

      IORoutines.writeStream(mp3Pointer, "us-ascii",
          new FileOutputStream(new File(getBaseStoragePath() + "/" + datePath + "/" + playlistFile)));
      reportChange(getBaseStoragePath() + "/" + datePath + "/" + mpegURLFile);
      reportChange(getBaseStoragePath() + "/" + datePath + "/" + playlistFile);
    }
    catch (Throwable e) {
      logger.error("MediaHandlerMp3.execute: " + e.toString());

      throw new MediaFailure(e);
    }
  }
  
  public String getDescr(Entity mediaType) {
    return "mp3";
  }
}



