/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.module;

import mir.log.LoggerWrapper;
import mir.module.AbstractModule;
import mir.util.JDBCStringRoutines;
import mircoders.global.MirGlobal;
import mircoders.storage.DatabaseContent;

/**
 *
 * <p>Title: ModuleContent</p>
 * <p>Description: helper routines to manage articles</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class ModuleContent extends AbstractModule
{
  static LoggerWrapper logger = new LoggerWrapper("Module.Content");

  public ModuleContent() {
    super(DatabaseContent.getInstance());
  }

  public void expireArticleLock(String anId, String aUserId) {
    if (!MirGlobal.isUserLoggedIn(anId)) {
      unlockArticle(anId, aUserId, false);
    }
  }

  /**
   * Locks an article. Forced (that is, without checking wether it's already locked)
   * if necessary.
   *
   * @param anId The article number
   * @param aUserId The locking user
   * @param aForce Should it e forced?
   * @return
   */
  public boolean lockArticle(String anId, String aUserId, boolean aForce) {
    try {
      String query =
          "update content set to_locking_user=" + JDBCStringRoutines.escapeStringLiteral(aUserId) +
          "  where id = " + JDBCStringRoutines.escapeStringLiteral(anId);

      if (!aForce)
        query = query + " and to_locking_user is null";

      return database.executeUpdate(query) > 0;
    }
    catch (Throwable t) {
      return false;
    }
  }

  /**
   * Unlocks an article. Forced (that is, without checking wether it's locked by
   *     the person that unlocks it) if necessary.
   *
   * @param anId The article number
   * @param aUserId The user that unlocks
   * @param aForce Should it e forced?
   * @return <code>true</code> if successfull, <code>false</code> if not
   */
  public boolean unlockArticle(String anId, String aUserId, boolean aForce)  {
    try {
      String query =
          "update content set to_locking_user=null" +
          "  where id = " + JDBCStringRoutines.escapeStringLiteral(anId);

      if (!aForce)
        query = query + " and to_locking_user = "+JDBCStringRoutines.escapeStringLiteral(aUserId);

      return database.executeUpdate(query) > 0;
    }
    catch (Throwable t) {
      return false;
    }
  }

  /**
   * Returns the id of the locking user of an article. <code>null</code> otherwise.
   *
   * @param anId
   * @return
   */
  public String queryArticleLock(String anId)  {
    try {
      String result = database.executeFreeSingleValueSql("select to_locking_user from content where id = " + JDBCStringRoutines.escapeStringLiteral(anId));

      if (result!=null && !MirGlobal.isUserLoggedIn(result)) {
        expireArticleLock(anId, result);
        result = null;
      }

      return result;
    }
    catch (Throwable t) {
      return null;
    }
  }
}


