/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.producer;

import mir.log.LoggerWrapper;
import mir.producer.AbstractProducerNode;
import mir.producer.ProducerFailure;
import mir.util.ExceptionRoutines;
import mir.util.ParameterExpander;
import org.apache.xmlrpc.XmlRpcClient;

import java.util.*;


public class RadicalendarProducerNode extends AbstractProducerNode {
  private String key;
  private String groupname;
  private String gmtoffset;


    public RadicalendarProducerNode(String aKey, String aGroupName, String aGMTOffset) {
      key = aKey;
      groupname = aGroupName;
      gmtoffset = aGMTOffset;
    
  }

  public void produce(Map aValueMap, String aVerb, LoggerWrapper aLogger) throws ProducerFailure {
    try {
      String expandedGroupName = ParameterExpander.expandExpression( aValueMap, groupname);
      String expandedGMTOffset = ParameterExpander.expandExpression( aValueMap, gmtoffset);
      
      XmlRpcClient xmlrpc = new XmlRpcClient ("http://radicalendar.org:80/xmlresponderweek.php");
      Vector params = new Vector();
      params.add(expandedGroupName+":"+expandedGMTOffset); //(shortname from calendar.getGroups)+gmtoffset in minutes
      Hashtable result = (Hashtable) xmlrpc.execute ("calendar.getWeek", params);
    

      
      ArrayList thisWeekEventsList=new ArrayList();

      ArrayList day_keys=new ArrayList();
      
      for (Enumeration e = result.keys() ; e.hasMoreElements() ; ){
        day_keys.add((String) e.nextElement());
      }  
      Collections.sort(day_keys);
      
      for(Iterator i = day_keys.iterator(); i.hasNext(); ){
	Hashtable day = (Hashtable) result.get((String) i.next());
	ArrayList order_keys = new ArrayList();
	
	for (Enumeration ee = day.keys() ; ee.hasMoreElements() ; ){
	  order_keys.add((String) ee.nextElement());
	}
	Collections.sort(order_keys);
	  
	for (Iterator ii = order_keys.iterator(); ii.hasNext();){
	  Hashtable event = (Hashtable) day.get((String) ii.next());
	  thisWeekEventsList.add(event); 
	}
      }


      ParameterExpander.setValueForKey(
				       aValueMap,
				       key,
				       thisWeekEventsList
				       );
    }   
    catch (Throwable t) {
      Throwable s = ExceptionRoutines.traceCauseException(t);
      aLogger.error("Error while accessing radicalendar: " + s.getClass().getName()+","+ s.getMessage());
    }
  }
}
